#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
require('module-alias/register');
var commander_1 = require("commander");
var chalk_1 = tslib_1.__importDefault(require("chalk"));
var package_json_1 = require("./package.json");
var functions_1 = require("@app/functions");
var inquirer_1 = tslib_1.__importDefault(require("inquirer"));
var utis_1 = require("@app/utis");
var program = new commander_1.Command();
program.version(package_json_1.version);
console.log("\nWelcome to " + chalk_1.default.magenta('create-nrd-app') + "\n");
program
    .arguments('<projectName>')
    .option('-n, --useNpm', 'Use npm to install dependencies')
    .action(function (projectName, _a) {
    var useNpm = _a.useNpm;
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var dockerComposeFilesInJson;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    console.log("Starting " + chalk_1.default.cyan('create-nrd-app') + "...");
                    console.log("Resolving " + chalk_1.default.cyan('server') + "...");
                    return [4, functions_1.generateFolder(projectName)];
                case 1:
                    _b.sent();
                    return [4, functions_1.generateFolder(projectName + "/server")];
                case 2:
                    _b.sent();
                    return [4, functions_1.generateFolder(projectName + "/server/src")];
                case 3:
                    _b.sent();
                    return [4, functions_1.generatePackageJson(projectName)];
                case 4:
                    _b.sent();
                    return [4, inquirer_1.default.prompt({
                            type: 'list',
                            message: 'Choose file type for docker-compose:',
                            name: 'dockerComposeFilesInJson',
                            choices: [
                                { name: 'JSON', value: true },
                                { name: 'YML', value: false },
                            ],
                        })];
                case 5:
                    dockerComposeFilesInJson = (_b.sent()).dockerComposeFilesInJson;
                    return [4, functions_1.generateDockerCompose(projectName, 'production', dockerComposeFilesInJson)];
                case 6:
                    _b.sent();
                    return [4, functions_1.generateDockerCompose(projectName, 'development', dockerComposeFilesInJson)];
                case 7:
                    _b.sent();
                    return [4, functions_1.generateTsConfigJson(projectName)];
                case 8:
                    _b.sent();
                    return [4, functions_1.generateServerPackageJson(projectName)];
                case 9:
                    _b.sent();
                    utis_1.spinnies.add('create-server-package-json', { text: 'Creating server/package.json' });
                    utis_1.spinnies.succeed('create-server-package-json', { text: 'server/package.json created!' });
                    return [4, functions_1.generateServerIndex(projectName)];
                case 10:
                    _b.sent();
                    return [4, functions_1.generateEslintConfig(projectName)];
                case 11:
                    _b.sent();
                    return [4, functions_1.generatePrettierConfig(projectName)];
                case 12:
                    _b.sent();
                    return [4, functions_1.generateGlobalSetupJest(projectName)];
                case 13:
                    _b.sent();
                    return [4, functions_1.generateGlobalTeardownJest(projectName)];
                case 14:
                    _b.sent();
                    return [4, functions_1.generateDockerIgnore(projectName + "/server/.dockerignore")];
                case 15:
                    _b.sent();
                    return [4, functions_1.generateGitIgnore(projectName + "/server/.gitignore")];
                case 16:
                    _b.sent();
                    return [4, functions_1.generateDockerfile(projectName, 'production', 'server')];
                case 17:
                    _b.sent();
                    return [4, functions_1.generateDockerfile(projectName, 'development', 'server')];
                case 18:
                    _b.sent();
                    return [4, functions_1.installServerPackages(projectName, useNpm)];
                case 19:
                    _b.sent();
                    console.log("Resolving " + chalk_1.default.cyan('client') + "...");
                    return [4, functions_1.executeCreateReactApp(projectName, useNpm)];
                case 20:
                    _b.sent();
                    return [4, functions_1.generateFolder(projectName + "/client/conf")];
                case 21:
                    _b.sent();
                    return [4, functions_1.generateFolder(projectName + "/client/conf/conf.d")];
                case 22:
                    _b.sent();
                    return [4, functions_1.generateNginxConfigFile(projectName)];
                case 23:
                    _b.sent();
                    return [4, functions_1.generateDockerIgnore(projectName + "/client/.dockerignore")];
                case 24:
                    _b.sent();
                    return [4, functions_1.generateDockerfile(projectName, 'production', 'client')];
                case 25:
                    _b.sent();
                    return [4, functions_1.generateDockerfile(projectName, 'development', 'client')];
                case 26:
                    _b.sent();
                    return [2];
            }
        });
    });
});
program.parse(process.argv);
