"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateNginxConfigFile = exports.executeCreateReactApp = exports.installServerPackages = exports.generateDockerfile = exports.generateGitIgnore = exports.generateDockerIgnore = exports.generateGlobalTeardownJest = exports.generateGlobalSetupJest = exports.generatePrettierConfig = exports.generateEslintConfig = exports.generateServerPackageJson = exports.generateServerIndex = exports.generateTsConfigJson = exports.generateDockerCompose = exports.generatePackageJson = exports.generateFolder = exports.getPackageVersions = void 0;
var tslib_1 = require("tslib");
var node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
var semver_1 = require("semver");
var fs_1 = tslib_1.__importStar(require("fs"));
var utis_1 = require("./utis");
var inquirer_1 = tslib_1.__importDefault(require("inquirer"));
var chalk_1 = tslib_1.__importDefault(require("chalk"));
var json_to_pretty_yaml_1 = tslib_1.__importDefault(require("json-to-pretty-yaml"));
var template_file_1 = require("template-file");
var latest_version_1 = tslib_1.__importDefault(require("latest-version"));
var spawn_async_1 = tslib_1.__importDefault(require("@expo/spawn-async"));
var path_1 = tslib_1.__importDefault(require("path"));
var serverPackages = {
    dependencies: [{ name: 'body-parser' }, { name: 'cors' }, { name: 'express' }, { name: 'compression' }],
    devDependencies: [
        { name: '@types/compression' },
        { name: '@types/express' },
        { name: '@types/jest' },
        { name: '@types/cors' },
        { name: '@types/node' },
        { name: '@types/supertest' },
        { name: '@typescript-eslint/eslint-plugin' },
        { name: '@typescript-eslint/parser' },
        { name: 'eslint' },
        { name: 'eslint-config-airbnb-base' },
        { name: 'eslint-config-prettier' },
        { name: 'eslint-plugin-import' },
        { name: 'eslint-plugin-prettier' },
        { name: 'husky' },
        { name: 'jest' },
        { name: 'lint-staged' },
        { name: 'nodemon' },
        { name: 'prettier' },
        { name: 'supertest' },
        { name: 'ts-jest' },
        { name: 'ts-node' },
        { name: 'typescript' },
    ],
};
function getLastPackageVersions(versions) {
    return Object.keys(versions)
        .filter(function (version) { return /^([0-9]+)\.([0-9]+)\.([0-9]+)$/.test(version); })
        .sort(function (a, b) {
        if (a === b) {
            return 0;
        }
        return semver_1.lt(a, b) ? 1 : -1;
    })
        .slice(0, 20);
}
function getPackageVersions(packageName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var response, versions;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, node_fetch_1.default("https://registry.npmjs.org/" + packageName, {
                        headers: {
                            Accept: 'application/vnd.npm.install-v1+json',
                        },
                    })];
                case 1:
                    response = _a.sent();
                    return [4, response.json()];
                case 2:
                    versions = (_a.sent()).versions;
                    return [2, {
                            versions: getLastPackageVersions(versions),
                        }];
            }
        });
    });
}
exports.getPackageVersions = getPackageVersions;
function generateFolder(folderPath) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    utis_1.spinnies.add('generate-folder', { text: "Creating " + folderPath });
                    if (fs_1.default.existsSync(process.cwd() + "/" + folderPath)) {
                        utis_1.spinnies.fail('generate-folder', { text: 'Folder already exist, use another name! Aborting...' });
                        return [2, process.exit(1)];
                    }
                    return [4, fs_1.promises.mkdir(process.cwd() + "/" + folderPath)];
                case 1:
                    _a.sent();
                    return [2, utis_1.spinnies.succeed('generate-folder', { text: "Folder " + folderPath + " created!" })];
            }
        });
    });
}
exports.generateFolder = generateFolder;
function generatePackageJson(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var values;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, inquirer_1.default.prompt([
                        {
                            name: 'name',
                            message: 'What is the project name?',
                            default: projectName,
                        },
                        {
                            name: 'version',
                            message: 'What is the project version?',
                            default: '0.1.0',
                        },
                        {
                            name: 'description',
                            message: "Describe your project here " + chalk_1.default.yellow('(optional)') + ":",
                            default: null,
                        },
                        {
                            name: 'author',
                            message: "What is the project author? " + chalk_1.default.yellow('(optional)'),
                            default: null,
                        },
                        {
                            name: 'license',
                            message: 'What is the project license?',
                            default: 'ISC',
                        },
                    ])];
                case 1:
                    values = _a.sent();
                    utis_1.spinnies.add('create-package-json', { text: 'Creating package.json' });
                    fs_1.promises.writeFile(process.cwd() + "/" + projectName + "/package.json", JSON.stringify(values, null, 2));
                    return [2, utis_1.spinnies.succeed('create-package-json', { text: 'package.json created!' })];
            }
        });
    });
}
exports.generatePackageJson = generatePackageJson;
function getDockerComposeData(projectName, environment, isProduction) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/docker-compose.template.mustache"), {
                    projectName: projectName,
                    environment: environment,
                    dockerfile: isProduction ? 'Dockerfile' : 'dev.Dockerfile',
                    appPort: isProduction ? 80 : 3000,
                    appContainerPort: isProduction ? '3000:80' : '3000:3000',
                })];
        });
    });
}
function generateDockerCompose(projectName, environment, dockerComposeFilesInJson) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var isProduction, fileData;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    isProduction = !(environment === 'development');
                    utis_1.spinnies.add("create-" + environment + "-docker", {
                        text: "Creating docker-compose" + (!isProduction ? '.dev' : '') + (dockerComposeFilesInJson ? '.json' : '.yml'),
                    });
                    return [4, getDockerComposeData(projectName, environment, isProduction)];
                case 1:
                    fileData = _a.sent();
                    return [4, fs_1.promises.writeFile(process.cwd() + "/" + projectName + "/docker-compose" + (!isProduction ? '.dev' : '') + (dockerComposeFilesInJson ? '.json' : '.yml'), dockerComposeFilesInJson ? fileData : json_to_pretty_yaml_1.default.stringify(JSON.parse(fileData)))];
                case 2:
                    _a.sent();
                    return [2, utis_1.spinnies.succeed("create-" + environment + "-docker", {
                            text: "docker-compose" + (!isProduction ? '.dev' : '') + (dockerComposeFilesInJson ? '.json' : '.yml') + " created!",
                        })];
            }
        });
    });
}
exports.generateDockerCompose = generateDockerCompose;
function getTsConfigData() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/tsconfig.template.mustache"))];
        });
    });
}
function generateTsConfigJson(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    utis_1.spinnies.add('create-ts-config', { text: 'Creating server/tsconfig.json' });
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + projectName + "/server/tsconfig.json"];
                    return [4, getTsConfigData()];
                case 1: return [4, _b.apply(_a, _c.concat([_d.sent()]))];
                case 2:
                    _d.sent();
                    return [2, utis_1.spinnies.succeed('create-ts-config', { text: 'server/tsconfig.json created!' })];
            }
        });
    });
}
exports.generateTsConfigJson = generateTsConfigJson;
function getDependenciesVersions() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var wantToSelectVersions;
        var _this = this;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, inquirer_1.default.prompt({
                        type: 'list',
                        message: 'Do you want to select the dependencies versions? (Selecting no, the last version will be selected)',
                        name: 'wantToSelectVersions',
                        choices: [
                            { name: 'Yes', value: true },
                            { name: 'No', value: false },
                        ],
                    })];
                case 1:
                    wantToSelectVersions = (_a.sent()).wantToSelectVersions;
                    if (!wantToSelectVersions) {
                        return [2, serverPackages.dependencies.reduce(function (promiseAccumulator, dependency) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var accumulator, _a, _b;
                                var _c;
                                return tslib_1.__generator(this, function (_d) {
                                    switch (_d.label) {
                                        case 0: return [4, promiseAccumulator];
                                        case 1:
                                            accumulator = _d.sent();
                                            _a = [tslib_1.__assign({}, accumulator)];
                                            _c = {};
                                            _b = dependency.name;
                                            return [4, latest_version_1.default(dependency.name)];
                                        case 2: return [2, tslib_1.__assign.apply(void 0, _a.concat([(_c[_b] = _d.sent(), _c)]))];
                                    }
                                });
                            }); }, Promise.resolve({}))];
                    }
                    return [2, serverPackages.dependencies.reduce(function (promiseAccumulator, dependency) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                            var accumulator, selectedVersion, _a, _b, _c;
                            var _d;
                            return tslib_1.__generator(this, function (_e) {
                                switch (_e.label) {
                                    case 0: return [4, promiseAccumulator];
                                    case 1:
                                        accumulator = _e.sent();
                                        _b = (_a = inquirer_1.default).prompt;
                                        _c = {
                                            type: 'list',
                                            message: "Select version for " + chalk_1.default.green(dependency.name),
                                            name: 'selectedVersion'
                                        };
                                        return [4, getPackageVersions(dependency.name)];
                                    case 2: return [4, _b.apply(_a, [(_c.choices = (_e.sent()).versions.map(function (version) { return ({ name: version, value: version }); }),
                                                _c)])];
                                    case 3:
                                        selectedVersion = (_e.sent()).selectedVersion;
                                        return [2, tslib_1.__assign(tslib_1.__assign({}, accumulator), (_d = {}, _d[dependency.name] = selectedVersion, _d))];
                                }
                            });
                        }); }, Promise.resolve({}))];
            }
        });
    });
}
function getDevDependenciesVersions() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var wantToSelectVersions;
        var _this = this;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, inquirer_1.default.prompt({
                        type: 'list',
                        message: 'Do you want to select the devDependencies versions? (Selecting no, the last version will be selected)',
                        name: 'wantToSelectVersions',
                        choices: [
                            { name: 'Yes', value: true },
                            { name: 'No', value: false },
                        ],
                    })];
                case 1:
                    wantToSelectVersions = (_a.sent()).wantToSelectVersions;
                    if (!wantToSelectVersions) {
                        return [2, serverPackages.devDependencies.reduce(function (promiseAccumulator, dependency) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var accumulator, _a, _b;
                                var _c;
                                return tslib_1.__generator(this, function (_d) {
                                    switch (_d.label) {
                                        case 0: return [4, promiseAccumulator];
                                        case 1:
                                            accumulator = _d.sent();
                                            _a = [tslib_1.__assign({}, accumulator)];
                                            _c = {};
                                            _b = dependency.name;
                                            return [4, latest_version_1.default(dependency.name)];
                                        case 2: return [2, tslib_1.__assign.apply(void 0, _a.concat([(_c[_b] = _d.sent(), _c)]))];
                                    }
                                });
                            }); }, Promise.resolve({}))];
                    }
                    return [2, serverPackages.devDependencies.reduce(function (promiseAccumulator, dependency) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                            var accumulator, selectedVersion, _a, _b, _c;
                            var _d;
                            return tslib_1.__generator(this, function (_e) {
                                switch (_e.label) {
                                    case 0: return [4, promiseAccumulator];
                                    case 1:
                                        accumulator = _e.sent();
                                        _b = (_a = inquirer_1.default).prompt;
                                        _c = {
                                            type: 'list',
                                            message: "Select version for " + chalk_1.default.green(dependency.name),
                                            name: 'selectedVersion'
                                        };
                                        return [4, getPackageVersions(dependency.name)];
                                    case 2: return [4, _b.apply(_a, [(_c.choices = (_e.sent()).versions.map(function (version) { return ({ name: version, value: version }); }),
                                                _c)])];
                                    case 3:
                                        selectedVersion = (_e.sent()).selectedVersion;
                                        return [2, tslib_1.__assign(tslib_1.__assign({}, accumulator), (_d = {}, _d[dependency.name] = selectedVersion, _d))];
                                }
                            });
                        }); }, Promise.resolve({}))];
            }
        });
    });
}
function getServerPackageJsonData(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c, _d, _e, _f, _g;
        return tslib_1.__generator(this, function (_h) {
            switch (_h.label) {
                case 0:
                    _a = template_file_1.renderTemplateFile;
                    _b = [path_1.default.join(__dirname, '..', "templates/api.packageJson.template.mustache")];
                    _c = {
                        projectName: projectName
                    };
                    _e = (_d = JSON).stringify;
                    return [4, getDependenciesVersions()];
                case 1:
                    _c.dependencies = _e.apply(_d, [_h.sent(), null, 2]);
                    _g = (_f = JSON).stringify;
                    return [4, getDevDependenciesVersions()];
                case 2: return [2, _a.apply(void 0, _b.concat([(_c.devDependencies = _g.apply(_f, [_h.sent(), null, 2]),
                            _c)]))];
            }
        });
    });
}
function generateServerPackageJson(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + projectName + "/server/package.json"];
                    return [4, getServerPackageJsonData(projectName)];
                case 1: return [2, _b.apply(_a, _c.concat([_d.sent()]))];
            }
        });
    });
}
exports.generateServerPackageJson = generateServerPackageJson;
function getServerIndexExpress() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/api.index.template.mustache"))];
        });
    });
}
function generateServerIndex(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    utis_1.spinnies.add('create-index-server', { text: 'Creating server/src/server.ts' });
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + projectName + "/server/src/server.ts"];
                    return [4, getServerIndexExpress()];
                case 1: return [4, _b.apply(_a, _c.concat([_d.sent()]))];
                case 2:
                    _d.sent();
                    return [2, utis_1.spinnies.succeed('create-index-server', { text: 'server/src/server.ts created!' })];
            }
        });
    });
}
exports.generateServerIndex = generateServerIndex;
function getEslintConfigData() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/api.eslint.template.mustache"))];
        });
    });
}
function generateEslintConfig(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    utis_1.spinnies.add('create-eslint-config', { text: 'Creating server/.eslintrc.json' });
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + projectName + "/server/.eslintrc.json"];
                    return [4, getEslintConfigData()];
                case 1: return [4, _b.apply(_a, _c.concat([_d.sent()]))];
                case 2:
                    _d.sent();
                    return [2, utis_1.spinnies.succeed('create-eslint-config', { text: 'server/.eslintrc.json created!' })];
            }
        });
    });
}
exports.generateEslintConfig = generateEslintConfig;
function getPrettierConfigData() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/api.prettier.template.mustache"))];
        });
    });
}
function generatePrettierConfig(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    utis_1.spinnies.add('create-prettier-config', { text: 'Creating server/.prettierrc' });
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + projectName + "/server/.prettierrc"];
                    return [4, getPrettierConfigData()];
                case 1: return [4, _b.apply(_a, _c.concat([_d.sent()]))];
                case 2:
                    _d.sent();
                    return [2, utis_1.spinnies.succeed('create-prettier-config', { text: 'server/.prettierrc created!' })];
            }
        });
    });
}
exports.generatePrettierConfig = generatePrettierConfig;
function getGlobalSetupJestData() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/api.jest-setup.template.mustache"))];
        });
    });
}
function generateGlobalSetupJest(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    utis_1.spinnies.add('create-setup-jest-config', { text: 'Creating server/global.setup.js' });
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + projectName + "/server/global.setup.js"];
                    return [4, getGlobalSetupJestData()];
                case 1: return [4, _b.apply(_a, _c.concat([_d.sent()]))];
                case 2:
                    _d.sent();
                    return [2, utis_1.spinnies.succeed('create-setup-jest-config', { text: 'server/global.setup.js created!' })];
            }
        });
    });
}
exports.generateGlobalSetupJest = generateGlobalSetupJest;
function getGlobalTeardownJestData() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/api.jest-teardown.template.mustache"))];
        });
    });
}
function generateGlobalTeardownJest(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    utis_1.spinnies.add('create-teardown-jest-config', { text: 'Creating server/global.teardown.js' });
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + projectName + "/server/global.teardown.js"];
                    return [4, getGlobalTeardownJestData()];
                case 1: return [4, _b.apply(_a, _c.concat([_d.sent()]))];
                case 2:
                    _d.sent();
                    return [2, utis_1.spinnies.succeed('create-teardown-jest-config', { text: 'server/global.teardown.js created!' })];
            }
        });
    });
}
exports.generateGlobalTeardownJest = generateGlobalTeardownJest;
function getDockerignoreData() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/dockerignore.template.mustache"))];
        });
    });
}
function generateDockerIgnore(folderPath) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    utis_1.spinnies.add("create-" + folderPath + "-dockerignore", { text: "Creating " + folderPath });
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + folderPath];
                    return [4, getDockerignoreData()];
                case 1: return [4, _b.apply(_a, _c.concat([_d.sent()]))];
                case 2:
                    _d.sent();
                    return [2, utis_1.spinnies.succeed("create-" + folderPath + "-dockerignore", { text: folderPath + " created!" })];
            }
        });
    });
}
exports.generateDockerIgnore = generateDockerIgnore;
function getGitIgnoreData() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/gitignore.template.mustache"))];
        });
    });
}
function generateGitIgnore(folderPath) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    utis_1.spinnies.add("create-" + folderPath + "-gitignore", { text: "Creating " + folderPath });
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + folderPath];
                    return [4, getGitIgnoreData()];
                case 1: return [4, _b.apply(_a, _c.concat([_d.sent()]))];
                case 2:
                    _d.sent();
                    return [2, utis_1.spinnies.succeed("create-" + folderPath + "-gitignore", { text: folderPath + " created!" })];
            }
        });
    });
}
exports.generateGitIgnore = generateGitIgnore;
function getDockerfileData(isProduction, isServer) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/" + (isServer ? 'api' : 'client') + "." + (isProduction ? 'Dockerfile' : 'devDockerfile') + ".template.mustache"))];
        });
    });
}
function generateDockerfile(projectName, environment, app) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var isProduction, isServer, fileName, _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    isProduction = !(environment === 'development');
                    isServer = !!(app === 'server');
                    fileName = isProduction ? 'Dockerfile' : 'dev.Dockerfile';
                    utis_1.spinnies.add("create-" + app + "-" + environment + "-dockerfile", { text: "Creating " + app + "/" + fileName });
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + projectName + "/" + app + "/" + fileName];
                    return [4, getDockerfileData(isProduction, isServer)];
                case 1: return [4, _b.apply(_a, _c.concat([_d.sent()]))];
                case 2:
                    _d.sent();
                    return [2, utis_1.spinnies.succeed("create-" + app + "-" + environment + "-dockerfile", { text: app + "/" + fileName + " created!" })];
            }
        });
    });
}
exports.generateDockerfile = generateDockerfile;
function installServerPackages(projectName, isNpm) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    utis_1.spinnies.add('install-packages', { text: "Installing Server packages with " + chalk_1.default.yellow(isNpm ? 'npm' : 'yarn') });
                    return [4, spawn_async_1.default('cd', ["./" + projectName + "/server", '&&', isNpm ? 'npm' : 'yarn', 'install', '&&', 'cd', '../..'], { shell: true })];
                case 1:
                    _a.sent();
                    return [2, utis_1.spinnies.succeed('install-packages', { text: 'Server packages installed' })];
            }
        });
    });
}
exports.installServerPackages = installServerPackages;
function executeCreateReactApp(projectName, isNpm) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var error_1;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    utis_1.spinnies.add('execute-create-react-app', {
                        text: "Executing create-react-app with " + chalk_1.default.yellow(isNpm ? 'npx create-react-app' : 'yarn create react-app'),
                    });
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4, spawn_async_1.default('cd', [
                            "./" + projectName,
                            '&&',
                            "" + (isNpm ? 'npm init react-app client --use-npm' : 'yarn create react-app client'),
                            '&&',
                            'cd',
                            '..',
                        ], { shell: true })];
                case 2:
                    _a.sent();
                    return [2, utis_1.spinnies.succeed('execute-create-react-app', { text: 'create-react-app finished!' })];
                case 3:
                    error_1 = _a.sent();
                    utis_1.spinnies.fail('execute-create-react-app', { text: 'create-react-app failed! :( Aborting...' });
                    return [2, process.exit(1)];
                case 4: return [2];
            }
        });
    });
}
exports.executeCreateReactApp = executeCreateReactApp;
function getNginxConfigFileData(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2, template_file_1.renderTemplateFile(path_1.default.join(__dirname, '..', "templates/client.nginxconf.template.mustache"), { projectName: projectName })];
        });
    });
}
function generateNginxConfigFile(projectName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    utis_1.spinnies.add('create-client-nginx-config-file', { text: 'Creating client/conf/conf.d/default.conf' });
                    _b = (_a = fs_1.default.promises).writeFile;
                    _c = [process.cwd() + "/" + projectName + "/client/conf/conf.d/default.conf"];
                    return [4, getNginxConfigFileData(projectName)];
                case 1: return [4, _b.apply(_a, _c.concat([_d.sent()]))];
                case 2:
                    _d.sent();
                    return [2, utis_1.spinnies.succeed('create-client-nginx-config-file', { text: 'client/conf/conf.d/default.conf created!' })];
            }
        });
    });
}
exports.generateNginxConfigFile = generateNginxConfigFile;
